import config from "config";
import { Events, NewsChannel, type VoiceState } from "discord.js";
import client from "lib/client";
import EventEmitter from "node:events";
import { PomodoroService } from "actions/pomodoro/pomodoro.service";

export default class PomodoroController extends EventEmitter {
    private pomodoroService: PomodoroService;
    private activePomodoros = new Set<string>();
    private pomodoroChannels = [config.discord.vchannelIdPomodoro25, config.discord.vchannelIdPomodoro50];

    constructor() {
        super();
        this.pomodoroService = new PomodoroService();

        client.on(Events.VoiceStateUpdate, async (oldState, newState) => {
            const userId = newState.id;

            const joinedPomodoroVC = newState.channelId != null && this.pomodoroChannels.includes(newState.channelId) &&
                oldState.channelId !== newState.channelId;

            const leftPomodoroVC = oldState.channelId != null && this.pomodoroChannels.includes(oldState.channelId) &&
                newState.channelId !== config.discord.vchannelIdPomodoro25;

            if (joinedPomodoroVC && !this.activePomodoros.has(userId)) {
                const member = newState.member;
                const vchannel = newState.channel;
                if (!member || !vchannel) return;

                this.activePomodoros.add(userId);
                this.pomodoroService.startPomodoroLoop(member, vchannel);
            }

            if (leftPomodoroVC && this.activePomodoros.has(userId)) {
                this.pomodoroService.stopPomodoro(userId);
                this.activePomodoros.delete(userId);
            }
        });
    }

}
